/******************************************************************************
**
**      Filename:       mmap.c
**
**      Purpose:        Emulate the mmap() device driver call provided on
**                      some other versions of UNIX.
**
**      Functions:
**
**      1.1
**
******************************************************************************/
/*****************************************************************************
**
**        Copyright (c) 1990,1991 by Bit 3 Computer Corporation.
**                     All Rights Reserved.
**              License governs use and distribution.
**
*****************************************************************************/

#ifndef LINT
#ifdef __STDC__

  static const char revcntrl[] = "@(#)"__FILE__"  1.1";

#else   /* __STDC__ */

  static char revcntrl[] = "@(#) $Workfile:   mmap.c  $  1.1";

#endif  /* __STDC__ */
#endif  /* LINT */

#include    <stddef.h>
#include    <sys/types.h>
#include    <assert.h>

#include    <sys/mman.h>
#include    <sys/btio.h>

#ifdef  __hpux
#include    <sys/param.h>
#endif  /* __hpux */

/******************************************************************************
**
**      Function:    mmap()
**
**      Purpose:     Map a section of application virtual address space
**                   to a device.
**
**      Args:        addr            address to start mapping at.
**                                   Must be NULL.
**                   len             size of address window to
**                                   create.
**                   prot            read/write protection on
**                                   memory window.
**                   flags           mapping flags, usually MAP_SHARED.
**                   fd              file descriptor.
**                   off             device-defined offset.
**
**      Returns:     First-word address of memory window created.
**
**      Notes:       It is not possible to exactly emulate the mmap()
**                   system call.
**
**                   The maximum size and offset for memory-mapped access
**                   to the device driver is configuration and system
**                   dependent.
** 
**                   Not all parameters will be used by all versions
**                   of UNIX.  All the parameters should be set to
**                   appropriate values for compatibility with other
**                   versions of UNIX.  The device offset and file
**                   descriptor are used on all versions of UNIX.
**
******************************************************************************/

#ifdef  __STDC__

  caddr_t mmap(caddr_t addr, size_t len, int prot, int flags, int fd, off_t off)

#else   /* __STDC__ */

  caddr_t mmap(addr, len, prot, flags, fd, off)
      caddr_t addr;
      size_t len;
      int prot;
      int flags;
      int fd;
      off_t off;

#endif  /* __STDC__ */

{
    bt_mmap_t mapper;

    assert(addr == NULL);
    assert(len > 0);
    assert(off >= 0);

    mapper.mem_p = addr;
    mapper.offset = off;
    mapper.length = len;
    mapper.protection = prot;   /* Recommended: (PROT_READ | PROT_WRITE) */
    mapper.mode = flags;    /* Recommended: (MAP_SHARED) */

    if (ioctl(fd, BIOC_MMAP, &mapper)) {
        return( (caddr_t) -1);
    }

    return(mapper.mem_p);
}


/******************************************************************************
**
**      Function:       munmap()
**
**      Purpose:        Remove mapping of a section of application virtual
**                      address space to a device.
**
**      Args:           addr            Address to remove mapping from.
**                                      Must not be NULL.
**                      len             Size of address window to remove.
**
**      Returns:        -1      If there was an error.
**                       0      Success.
**
**      Notes:          This routine is included for completeness.  Since
**                      the munmap() routine does not usually include a file
**                      descriptor parameter, it is not possible to call the
**                      device driver when unmapping the memory segment.
**
**                      Therefore, this routine always returns a success status.
**
******************************************************************************/

#ifdef  __STDC__

  int munmap(caddr_t addr, size_t len)

#else   /* __STDC__ */

  int munmap(addr, len)
    caddr_t addr;
    size_t len;

#endif  /* __STDC__ */

{
    assert(addr != NULL);
    assert(len > 0);

    return(0);
}

#ifdef  __hpux

/******************************************************************************
**
**      Function:       getpagesize()
**
**      Purpose:        Return the system page size (in bytes) used by the
**                      system's memory-management unit.
**
**      Args:           Void
**
**      Returns:        Size of page in bytes.
**
******************************************************************************/

#ifdef  __STDC__

  int getpagesize(void)

#else   /* __STDC__ */

  int getpagesize()

#endif  /* __STDC__ */

{
    return(NBPG);
}

#endif  /* __hpux */
